@@ -10,9 +10,9 @@ radius = np.linspace(0, 20, 100) * u.kpc
 for key in radial_distributions:
     model = radial_distributions[key]()
     if model.evolved:
-        linestyle = '-'
+        linestyle = "-"
     else:
-        linestyle = '--'
+        linestyle = "--"
     label = model.__class__.__name__
     x = radius.value
     y = normalize(model, 0, radius[-1].value)(radius.value)
@@ -20,7 +20,7 @@ for key in radial_distributions:
 
 plt.xlim(0, radius[-1].value)
 plt.ylim(0, 0.26)
-plt.xlabel('Galactocentric Distance [kpc]')
-plt.ylabel('Normalized Surface Density [kpc^-2]')
-plt.legend(prop={'size': 10})
+plt.xlabel("Galactocentric Distance [kpc]")
+plt.ylabel("Normalized Surface Density [kpc^-2]")
+plt.legend(prop={"size": 10})
 plt.show()
