@@ -1,4 +1,2 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-"""
-Astrophysical source and population models.
-"""
+"""Astrophysical source and population models."""
