@@ -2,15 +2,10 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
 import astropy.units as u
-from ....utils.testing import (
-    assert_quantity_allclose,
-    requires_data,
-    requires_dependency,
-)
+from ....utils.testing import assert_quantity_allclose, requires_data
 from .. import PrimaryFlux
 
 
-@requires_dependency("scipy")
 @requires_data("gammapy-extra")
 def test_primary_flux():
     with pytest.raises(ValueError):
@@ -18,5 +13,5 @@ def test_primary_flux():
 
     primflux = PrimaryFlux(channel="W", mDM=1 * u.TeV)
     actual = primflux.table_model(500 * u.GeV)
-    desired = 9.324355468682548e-05 / u.GeV
+    desired = 9.328234e-05 / u.GeV
     assert_quantity_allclose(actual, desired)
