@@ -4,7 +4,8 @@ import pytest
 import numpy as np
 from numpy.testing import assert_allclose
 from astropy import units as u
-from ...utils.testing import assert_quantity_allclose
+from astropy.time import Time
+from ...utils.testing import assert_quantity_allclose, assert_time_allclose
 from ...utils.testing import requires_data, requires_dependency
 from ...spectrum.models import (
     PowerLaw,
@@ -128,9 +129,10 @@ class TestFermi3FGLObject:
 
         assert len(flux_points.table) == 5
         assert "flux_ul" in flux_points.table.colnames
+        assert flux_points.sed_type == "flux"
 
-        desired = [8.174943e-03, 7.676263e-04, 6.119782e-05, 3.350906e-06, 1.308784e-08]
-        assert_allclose(flux_points.table["dnde"].data, desired, rtol=1e-5)
+        desired = [1.645888e-06, 5.445407e-07, 1.255338e-07, 2.545524e-08, 2.263189e-09]
+        assert_allclose(flux_points.table["flux"].data, desired, rtol=1e-5)
 
     def test_flux_points_ul(self):
         source = self.cat["3FGL J0000.2-3738"]
@@ -152,8 +154,11 @@ class TestFermi3FGLObject:
             "flux_errn",
         ]
 
-        assert lc.time_min[0].fits == "2008-08-02T00:33:19.000(UTC)"
-        assert lc.time_max[0].fits == "2008-09-01T10:31:04.625(UTC)"
+        expected = Time(54680.02313657408, format="mjd", scale="utc")
+        assert_time_allclose(lc.time_min[0], expected)
+
+        expected = Time(54710.43824797454, format="mjd", scale="utc")
+        assert_time_allclose(lc.time_max[0], expected)
 
         assert table["flux"].unit == "cm-2 s-1"
         assert_allclose(table["flux"][0], 2.384e-06, rtol=1e-3)
@@ -238,7 +243,7 @@ class TestFermi2FHLObject:
 
         actual = flux_points.table["flux_ul"]
         desired = [np.nan, np.nan, 1.294092e-11] * u.Unit("cm-2 s-1")
-        assert_quantity_allclose(actual, desired, rtol=1E-3)
+        assert_quantity_allclose(actual, desired, rtol=1e-3)
 
 
 @requires_data("gammapy-extra")
@@ -301,8 +306,8 @@ class TestFermi3FHLObject:
         assert len(flux_points.table) == 5
         assert "flux_ul" in flux_points.table.colnames
 
-        desired = [5.124e-07, 7.370e-08, 9.044e-09, 7.681e-10, 4.307e-11]
-        assert_allclose(flux_points.table["dnde"].data, desired, rtol=1e-3)
+        desired = [5.169889e-09, 2.245024e-09, 9.243175e-10, 2.758956e-10, 6.684021e-11]
+        assert_allclose(flux_points.table["flux"].data, desired, rtol=1e-3)
 
     @pytest.mark.parametrize(
         "name", ["Crab Nebula", "3FHL J0534.5+2201", "3FGL J0534.5+2201i"]
