@@ -5,17 +5,15 @@ import numpy as np
 from collections import OrderedDict
 from astropy.coordinates import SkyCoord
 from astropy.units import Quantity
-from astropy.utils import lazyproperty
-from ..extern.six.moves import UserList  # pylint:disable=import-error
-from ..irf import EnergyDependentTablePSF, PSF3D, IRFStacker
+from astropy.time import Time
 from .event_list import EventListChecker
 from ..utils.testing import Checker
-from ..utils.energy import Energy
 from ..utils.fits import earth_location_from_dict
 from ..utils.table import table_row_to_dict
 from ..utils.time import time_ref_from_dict
+from .filters import ObservationFilter
 
-__all__ = ["ObservationCTA", "DataStoreObservation", "ObservationList"]
+__all__ = ["ObservationCTA", "DataStoreObservation", "Observations"]
 
 log = logging.getLogger(__name__)
 
@@ -128,9 +126,11 @@ class DataStoreObservation(object):
         Observation ID
     data_store : `~gammapy.data.DataStore`
         Data store
+    obs_filter : `~gammapy.data.ObservationFilter`, optional
+        Filter for the observation
     """
 
-    def __init__(self, obs_id, data_store):
+    def __init__(self, obs_id, data_store, obs_filter=None):
         # Assert that `obs_id` is available
         if obs_id not in data_store.obs_table["OBS_ID"]:
             raise ValueError("OBS_ID = {} not in obs index table.".format(obs_id))
@@ -139,6 +139,7 @@ class DataStoreObservation(object):
 
         self.obs_id = obs_id
         self.data_store = data_store
+        self.obs_filter = obs_filter or ObservationFilter()
 
     def __str__(self):
         ss = "Info for OBS_ID = {}\n".format(self.obs_id)
@@ -194,13 +195,23 @@ class DataStoreObservation(object):
 
     @property
     def events(self):
-        """Load `gammapy.data.EventList` object."""
-        return self.load(hdu_type="events")
+        """Load `gammapy.data.EventList` object and apply the filter."""
+        events = self.load(hdu_type="events")
+        return self.obs_filter.filter_events(events)
 
     @property
     def gti(self):
-        """Load `gammapy.data.GTI` object."""
-        return self.load(hdu_type="gti")
+        """Load `gammapy.data.GTI` object and apply the filter."""
+        try:
+            gti = self.load(hdu_type="gti")
+        except IndexError:
+            # For now we support data without GTI HDUs
+            # TODO: if GTI becomes required, we should drop this case
+            # CTA discussion in https://github.com/open-gamma-ray-astro/gamma-astro-data-formats/issues/20
+            # Added in Gammapy in https://github.com/gammapy/gammapy/pull/1908
+            gti = self.data_store.obs_table.create_gti(obs_id=self.obs_id)
+
+        return self.obs_filter.filter_gti(gti)
 
     @property
     def aeff(self):
@@ -222,13 +233,13 @@ class DataStoreObservation(object):
         """Load background object."""
         return self.load(hdu_type="bkg")
 
-    @lazyproperty
+    @property
     def obs_info(self):
         """Observation information (`~collections.OrderedDict`)."""
         row = self.data_store.obs_table.select_obs_id(obs_id=self.obs_id)[0]
         return table_row_to_dict(row)
 
-    @lazyproperty
+    @property
     def tstart(self):
         """Observation start time (`~astropy.time.Time`)."""
         met_ref = time_ref_from_dict(self.data_store.obs_table.meta)
@@ -236,7 +247,7 @@ class DataStoreObservation(object):
         time = met_ref + met
         return time
 
-    @lazyproperty
+    @property
     def tstop(self):
         """Observation stop time (`~astropy.time.Time`)."""
         met_ref = time_ref_from_dict(self.data_store.obs_table.meta)
@@ -244,15 +255,15 @@ class DataStoreObservation(object):
         time = met_ref + met
         return time
 
-    @lazyproperty
+    @property
     def observation_time_duration(self):
         """Observation time duration in seconds (`~astropy.units.Quantity`).
 
         The wall time, including dead-time.
         """
-        return Quantity(self.obs_info["ONTIME"], "second")
+        return self.gti.time_sum
 
-    @lazyproperty
+    @property
     def observation_live_time_duration(self):
         """Live-time duration in seconds (`~astropy.units.Quantity`).
 
@@ -261,9 +272,9 @@ class DataStoreObservation(object):
         Computed as ``t_live = t_observation * (1 - f_dead)``
         where ``f_dead`` is the dead-time fraction.
         """
-        return Quantity(self.obs_info["LIVETIME"], "second")
+        return self.gti.time_sum * (1 - self.observation_dead_time_fraction)
 
-    @lazyproperty
+    @property
     def observation_dead_time_fraction(self):
         """Dead-time fraction (float).
 
@@ -279,35 +290,35 @@ class DataStoreObservation(object):
         """
         return 1 - self.obs_info["DEADC"]
 
-    @lazyproperty
+    @property
     def pointing_radec(self):
         """Pointing RA / DEC sky coordinates (`~astropy.coordinates.SkyCoord`)."""
         lon, lat = self.obs_info["RA_PNT"], self.obs_info["DEC_PNT"]
         return SkyCoord(lon, lat, unit="deg", frame="icrs")
 
-    @lazyproperty
+    @property
     def pointing_altaz(self):
         """Pointing ALT / AZ sky coordinates (`~astropy.coordinates.SkyCoord`)."""
         alt, az = self.obs_info["ALT_PNT"], self.obs_info["AZ_PNT"]
         return SkyCoord(az, alt, unit="deg", frame="altaz")
 
-    @lazyproperty
+    @property
     def pointing_zen(self):
         """Pointing zenith angle sky (`~astropy.units.Quantity`)."""
         return Quantity(self.obs_info["ZEN_PNT"], unit="deg")
 
-    @lazyproperty
+    @property
     def target_radec(self):
         """Target RA / DEC sky coordinates (`~astropy.coordinates.SkyCoord`)."""
         lon, lat = self.obs_info["RA_OBJ"], self.obs_info["DEC_OBJ"]
         return SkyCoord(lon, lat, unit="deg", frame="icrs")
 
-    @lazyproperty
+    @property
     def observatory_earth_location(self):
         """Observatory location (`~astropy.coordinates.EarthLocation`)."""
         return earth_location_from_dict(self.obs_info)
 
-    @lazyproperty
+    @property
     def muoneff(self):
         """Observation muon efficiency."""
         return self.obs_info["MUONEFF"]
@@ -316,52 +327,6 @@ class DataStoreObservation(object):
         """Quick-look plots in a few panels."""
         raise NotImplementedError
 
-    def make_psf(self, position, energy=None, rad=None):
-        """Make energy-dependent PSF for a given source position.
-
-        Parameters
-        ----------
-        position : `~astropy.coordinates.SkyCoord`
-            Position at which to compute the PSF
-        energy : `~astropy.units.Quantity`
-            1-dim energy array for the output PSF.
-            If none is given, the energy array of the PSF from the observation is used.
-        rad : `~astropy.coordinates.Angle`
-            1-dim offset wrt source position array for the output PSF.
-            If none is given, the offset array of the PSF from the observation is used.
-
-        Returns
-        -------
-        psf : `~gammapy.irf.EnergyDependentTablePSF`
-            Energy dependent psf table
-        """
-        offset = position.separation(self.pointing_radec)
-
-        if energy is None:
-            energy = self.psf.to_energy_dependent_table_psf(theta=offset).energy
-
-        if rad is None:
-            rad = self.psf.to_energy_dependent_table_psf(theta=offset).rad
-
-        if isinstance(self.psf, PSF3D):
-            # PSF3D is a table PSF, so we use the native RAD binning by default
-            # TODO: should handle this via a uniform caller API
-            psf_value = self.psf.to_energy_dependent_table_psf(theta=offset).evaluate(
-                energy
-            )
-        else:
-            psf_value = self.psf.to_energy_dependent_table_psf(
-                theta=offset, rad=rad
-            ).evaluate(energy)
-
-        arf = self.aeff.data.evaluate(offset=offset, energy=energy)
-        exposure = arf * self.observation_live_time_duration
-
-        psf = EnergyDependentTablePSF(
-            energy=energy, rad=rad, exposure=exposure, psf_value=psf_value
-        )
-        return psf
-
     def to_observation_cta(self):
         """Convert to `~gammapy.data.ObservationCTA`.
 
@@ -370,7 +335,7 @@ class DataStoreObservation(object):
 
         Returns
         -------
-        obs : `~gammapy.data.ObservationCTA`
+        observation : `~gammapy.data.ObservationCTA`
             Observation
         """
         # maps the ObservationCTA class attributes to the DataStoreObservation properties
@@ -405,12 +370,24 @@ class DataStoreObservation(object):
         return checker.run(checks=checks)
 
 
-class ObservationList(UserList):
-    """List of `~gammapy.data.DataStoreObservation`.
+class Observations(object):
+    """Container class that holds a list of observations.
 
-    Could be extended to hold a more generic class of observations.
+    Parameters
+    ----------
+    obs_list : list
+        A list of `~gammapy.data.DataStoreObservation`
     """
 
+    def __init__(self, obs_list=None):
+        self._obs_list = obs_list or []
+
+    def __getitem__(self, key):
+        return self._obs_list[key]
+
+    def __len__(self):
+        return len(self._obs_list)
+
     def __str__(self):
         s = self.__class__.__name__ + "\n"
         s += "Number of observations: {}\n".format(len(self))
@@ -418,105 +395,6 @@ class ObservationList(UserList):
             s += str(obs)
         return s
 
-    def make_mean_psf(self, position, energy=None, rad=None):
-        """Compute mean energy-dependent PSF.
-
-        Parameters
-        ----------
-        position : `~astropy.coordinates.SkyCoord`
-            Position at which to compute the PSF
-        energy : `~astropy.units.Quantity`
-            1-dim energy array for the output PSF.
-            If none is given, the energy array of the PSF from the first
-            observation is used.
-        rad : `~astropy.coordinates.Angle`
-            1-dim offset wrt source position array for the output PSF.
-            If none is given, the energy array of the PSF from the first
-            observation is used.
-
-        Returns
-        -------
-        psf : `~gammapy.irf.EnergyDependentTablePSF`
-            Mean PSF
-        """
-        psf = self[0].make_psf(position, energy, rad)
-
-        if rad is None:
-            rad = psf.rad
-        if energy is None:
-            energy = psf.energy
-
-        exposure = psf.exposure
-        psf_value = psf.psf_value.T * psf.exposure
-
-        for obs in self[1:]:
-            psf = obs.make_psf(position, energy, rad)
-            exposure += psf.exposure
-            psf_value += psf.psf_value.T * psf.exposure
-
-        psf_value /= exposure
-        psf_tot = EnergyDependentTablePSF(
-            energy=energy, rad=rad, exposure=exposure, psf_value=psf_value.T
-        )
-        return psf_tot
-
-    def make_mean_edisp(
-        self,
-        position,
-        e_true,
-        e_reco,
-        low_reco_threshold=Energy(0.002, "TeV"),
-        high_reco_threshold=Energy(150, "TeV"),
-    ):
-        """Compute mean energy dispersion.
-
-        Compute the mean edisp of a set of observations j at a given position
-
-        The stacking is implemented in :func:`~gammapy.irf.IRFStacker.stack_edisp`
-
-        Parameters
-        ----------
-        position : `~astropy.coordinates.SkyCoord`
-            Position at which to compute the mean EDISP
-        e_true : `~gammapy.utils.energy.EnergyBounds`
-            True energy axis
-        e_reco : `~gammapy.utils.energy.EnergyBounds`
-            Reconstructed energy axis
-        low_reco_threshold : `~gammapy.utils.energy.Energy`
-            low energy threshold in reco energy, default 0.002 TeV
-        high_reco_threshold : `~gammapy.utils.energy.Energy`
-            high energy threshold in reco energy , default 150 TeV
-
-        Returns
-        -------
-        stacked_edisp : `~gammapy.irf.EnergyDispersion`
-            Stacked EDISP for a set of observation
-        """
-        list_aeff = []
-        list_edisp = []
-        list_livetime = []
-        list_low_threshold = [low_reco_threshold] * len(self)
-        list_high_threshold = [high_reco_threshold] * len(self)
-
-        for obs in self:
-            offset = position.separation(obs.pointing_radec)
-            list_aeff.append(obs.aeff.to_effective_area_table(offset, energy=e_true))
-            list_edisp.append(
-                obs.edisp.to_energy_dispersion(offset, e_reco=e_reco, e_true=e_true)
-            )
-            list_livetime.append(obs.observation_live_time_duration)
-
-        irf_stack = IRFStacker(
-            list_aeff=list_aeff,
-            list_edisp=list_edisp,
-            list_livetime=list_livetime,
-            list_low_threshold=list_low_threshold,
-            list_high_threshold=list_high_threshold,
-        )
-        irf_stack.stack_edisp()
-
-        return irf_stack.stacked_edisp
-
 
 class ObservationChecker(Checker):
     """Check an observation.
@@ -532,17 +410,17 @@ class ObservationChecker(Checker):
         "psf": "check_psf",
     }
 
-    def __init__(self, obs):
-        self.obs = obs
+    def __init__(self, observation):
+        self.observation = observation
 
     def _record(self, level="info", msg=None):
-        return {"level": level, "obs_id": self.obs.obs_id, "msg": msg}
+        return {"level": level, "obs_id": self.observation.obs_id, "msg": msg}
 
     def check_events(self):
         yield self._record(level="debug", msg="Starting events check")
 
         try:
-            events = self.obs.load("events")
+            events = self.observation.load("events")
         except Exception:
             yield self._record(level="warning", msg="Loading events failed")
             return
@@ -555,7 +433,7 @@ class ObservationChecker(Checker):
         yield self._record(level="debug", msg="Starting gti check")
 
         try:
-            gti = self.obs.load("gti")
+            gti = self.observation.load("gti")
         except Exception:
             yield self._record(level="warning", msg="Loading GTI failed")
             return
@@ -607,7 +485,7 @@ class ObservationChecker(Checker):
         yield self._record(level="debug", msg="Starting aeff check")
 
         try:
-            aeff = self.obs.load("aeff")
+            aeff = self.observation.load("aeff")
         except Exception:
             yield self._record(level="warning", msg="Loading aeff failed")
             return
@@ -632,7 +510,7 @@ class ObservationChecker(Checker):
         yield self._record(level="debug", msg="Starting edisp check")
 
         try:
-            edisp = self.obs.load("edisp")
+            edisp = self.observation.load("edisp")
         except Exception:
             yield self._record(level="warning", msg="Loading edisp failed")
             return
@@ -645,7 +523,7 @@ class ObservationChecker(Checker):
         yield self._record(level="debug", msg="Starting psf check")
 
         try:
-            psf = self.obs.load("psf")
+            psf = self.observation.load("psf")
         except Exception:
             yield self._record(level="warning", msg="Loading psf failed")
             return
