@@ -1,10 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import absolute_import, division, print_function, unicode_literals
 import pytest
-from ...data import DataStore
 from ...utils.testing import requires_data
-
-pytest.importorskip("scipy")
+from ...data import DataStore
 
 
 @pytest.fixture(scope="session")
@@ -81,16 +79,16 @@ def test_datastore_pa():
 
 
 @requires_data("gammapy-extra")
-def test_datastore_obslist(data_store):
+def test_datastore_get_observations(data_store):
     """Test loading data and IRF files via the DataStore"""
-    obslist = data_store.obs_list([23523, 23592])
-    assert obslist[0].obs_id == 23523
+    observations = data_store.get_observations([23523, 23592])
+    assert observations[0].obs_id == 23523
 
     with pytest.raises(ValueError):
-        data_store.obs_list([11111, 23592])
+        data_store.get_observations([11111, 23592])
 
-    obslist = data_store.obs_list([11111, 23523], skip_missing=True)
-    assert obslist[0].obs_id == 23523
+    observations = data_store.get_observations([11111, 23523], skip_missing=True)
+    assert observations[0].obs_id == 23523
 
 
 @requires_data("gammapy-extra")
@@ -129,4 +127,4 @@ class TestDataStoreChecker:
 
     def test_check_all(self):
         records = list(self.data_store.check())
-        assert len(records) == 48
+        assert len(records) == 32
