@@ -1,13 +1,12 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import absolute_import, division, print_function, unicode_literals
-from numpy.testing.utils import assert_allclose
+from numpy.testing import assert_allclose
 from astropy.convolution import Tophat2DKernel
-from ...utils.testing import requires_dependency, requires_data
+from ...utils.testing import requires_data
 from ...detect import compute_lima_image, compute_lima_on_off_image
 from ...maps import Map
 
 
-@requires_dependency("scipy")
 @requires_data("gammapy-extra")
 def test_compute_lima_image():
     """
@@ -26,7 +25,6 @@ def test_compute_lima_image():
     assert_allclose(result_lima["significance"].data[1, 1], 0.164, atol=1e-3)
 
 
-@requires_dependency("scipy")
 @requires_data("gammapy-extra")
 def test_compute_lima_on_off_image():
     """
