@@ -1,11 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import absolute_import, division, print_function, unicode_literals
-import numpy as np
 import pytest
+import numpy as np
 from astropy import units as u
 from astropy.modeling.models import Gaussian2D
 from astropy.coordinates import SkyCoord
-from ...utils.testing import assert_quantity_allclose, requires_dependency
+from ...utils.testing import assert_quantity_allclose
 from ...maps import WcsGeom, WcsNDMap
 from ...image import (
     measure_containment_radius,
@@ -22,7 +22,7 @@ def gaussian_image():
     binsz = 0.02
     sigma = 0.2
     geom = WcsGeom.create(npix=(201, 201), binsz=binsz, coordsys="GAL")
-    gauss = Gaussian2D(1. / (2 * np.pi * (sigma / binsz) ** 2), 0, 0, sigma, sigma)
+    gauss = Gaussian2D(1.0 / (2 * np.pi * (sigma / binsz) ** 2), 0, 0, sigma, sigma)
     coord = geom.get_coord().skycoord
     l = coord.data.lon.wrap_at("180d")
     b = coord.data.lat
@@ -56,7 +56,6 @@ def test_measure_containment(gaussian_image):
     assert_quantity_allclose(frac, ref, rtol=0.01)
 
 
-@requires_dependency("scipy")
 def test_measure_containment_radius(gaussian_image):
     """Test measure_containment_radius function"""
     position = SkyCoord(0, 0, frame="galactic", unit="deg")
