@@ -2,6 +2,7 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 from collections import OrderedDict
 import numpy as np
+from scipy.special import erf
 from astropy.io import fits
 from astropy.coordinates import Angle
 from astropy.units import Quantity
@@ -9,13 +10,9 @@ from astropy.table import Table
 from ..utils.energy import EnergyBounds, Energy
 from ..utils.scripts import make_path
 from ..utils.nddata import NDDataArray, BinnedDataAxis
-from ..utils.fits import energy_axis_to_ebounds, fits_table_to_table
-from ..utils.fits import fits_table_to_table, table_to_fits_table
+from ..utils.fits import energy_axis_to_ebounds
 
-__all__ = [
-    'EnergyDispersion',
-    'EnergyDispersion2D',
-]
+__all__ = ["EnergyDispersion", "EnergyDispersion2D"]
 
 
 class EnergyDispersion(object):
@@ -54,28 +51,37 @@ class EnergyDispersion(object):
     --------
     EnergyDispersion2D
     """
-    default_interp_kwargs = dict(bounds_error=False, fill_value=0,
-                                 method='nearest')
+
+    default_interp_kwargs = dict(bounds_error=False, fill_value=0, method="nearest")
     """Default Interpolation kwargs for `~NDDataArray`. Fill zeros and do not
     interpolate"""
 
-    def __init__(self, e_true_lo, e_true_hi, e_reco_lo, e_reco_hi, data,
-                 interp_kwargs=None, meta=None):
+    def __init__(
+        self,
+        e_true_lo,
+        e_true_hi,
+        e_reco_lo,
+        e_reco_hi,
+        data,
+        interp_kwargs=None,
+        meta=None,
+    ):
         if interp_kwargs is None:
             interp_kwargs = self.default_interp_kwargs
         axes = [
-            BinnedDataAxis(e_true_lo, e_true_hi,
-                           interpolation_mode='log', name='e_true'),
-            BinnedDataAxis(e_reco_lo, e_reco_hi,
-                           interpolation_mode='log', name='e_reco')
+            BinnedDataAxis(
+                e_true_lo, e_true_hi, interpolation_mode="log", name="e_true"
+            ),
+            BinnedDataAxis(
+                e_reco_lo, e_reco_hi, interpolation_mode="log", name="e_reco"
+            ),
         ]
-        self.data = NDDataArray(axes=axes, data=data,
-                                interp_kwargs=interp_kwargs)
+        self.data = NDDataArray(axes=axes, data=data, interp_kwargs=interp_kwargs)
         self.meta = OrderedDict(meta) if meta else OrderedDict()
 
     def __str__(self):
         ss = self.__class__.__name__
-        ss += '\n{}'.format(self.data)
+        ss += "\n{}".format(self.data)
         return ss
 
     def apply(self, data):
@@ -96,19 +102,22 @@ class EnergyDispersion(object):
             1-dim data array after multiplication with the energy dispersion matrix
         """
         if len(data) != self.e_true.nbins:
-            raise ValueError("Input size {} does not match true energy axis {}".format(
-                len(data), self.e_true.nbins))
+            raise ValueError(
+                "Input size {} does not match true energy axis {}".format(
+                    len(data), self.e_true.nbins
+                )
+            )
         return np.dot(data, self.data.data)
 
     @property
     def e_reco(self):
         """Reconstructed energy axis (`~gammapy.utils.nddata.BinnedDataAxis`)"""
-        return self.data.axis('e_reco')
+        return self.data.axis("e_reco")
 
     @property
     def e_true(self):
         """True energy axis (`~gammapy.utils.nddata.BinnedDataAxis`)"""
-        return self.data.axis('e_true')
+        return self.data.axis("e_true")
 
     @property
     def pdf_matrix(self):
@@ -130,8 +139,9 @@ class EnergyDispersion(object):
             High reco energy threshold
         """
         data = self.pdf_matrix.copy()
-        idx = np.where((self.e_reco.lo < lo_threshold) |
-                       (self.e_reco.hi > hi_threshold))
+        idx = np.where(
+            (self.e_reco.lo < lo_threshold) | (self.e_reco.hi > hi_threshold)
+        )
         data[:, idx] = 0
         return data
 
@@ -154,19 +164,71 @@ class EnergyDispersion(object):
         pdf_threshold : float, optional
             Zero suppression threshold
         """
-        migra = np.linspace(1. / 3, 3, 200)
+        migra = np.linspace(1.0 / 3, 3, 200)
         # A dummy offset axis (need length 2 for interpolation to work)
-        offset = Quantity([0, 1, 2], 'deg')
+        offset = Quantity([0, 1, 2], "deg")
 
         edisp = EnergyDispersion2D.from_gauss(
-            e_true=e_true, migra=migra,
-            sigma=sigma, bias=bias,
-            offset=offset, pdf_threshold=pdf_threshold,
+            e_true=e_true,
+            migra=migra,
+            sigma=sigma,
+            bias=bias,
+            offset=offset,
+            pdf_threshold=pdf_threshold,
         )
         return edisp.to_energy_dispersion(offset=offset[0], e_reco=e_reco)
 
     @classmethod
-    def from_hdulist(cls, hdulist, hdu1='MATRIX', hdu2='EBOUNDS'):
+    def from_diagonal_response(cls, e_true, e_reco=None):
+        """Create energy dispersion from a diagonal response, i.e. perfect energy resolution
+
+        This creates the matrix corresponding to a perfect energy response.
+        It contains ones where the e_true center is inside the e_reco bin.
+        It is a square diagonal matrix if e_true = e_reco.
+
+        This is useful in cases where code always applies an edisp,
+        but you don't want it to do anything.
+
+        Parameters
+        ----------
+        e_true, e_reco : `~astropy.units.Quantity`
+            Energy bounds for true and reconstructed energy axis
+
+        Examples
+        --------
+        If ``e_true`` equals ``e_reco``, you get a diagonal matrix::
+
+            e_true = [0.5, 1, 2, 4, 6] * u.TeV
+            edisp = EnergyDispersion.from_diagonal_response(e_true)
+            edisp.plot_matrix()
+
+        Example with different energy binnings::
+
+            e_true = [0.5, 1, 2, 4, 6] * u.TeV
+            e_reco = [2, 4, 6] * u.TeV
+            edisp = EnergyDispersion.from_diagonal_response(e_true, e_reco)
+            edisp.plot_matrix()
+        """
+        if e_reco is None:
+            e_reco = e_true
+
+        e_true_center = 0.5 * (e_true[1:] + e_true[:-1])
+        etrue_2d, ereco_lo_2d = np.meshgrid(e_true_center, e_reco[:-1])
+        etrue_2d, ereco_hi_2d = np.meshgrid(e_true_center, e_reco[1:])
+
+        data = np.logical_and(etrue_2d >= ereco_lo_2d, etrue_2d < ereco_hi_2d)
+        data = np.transpose(data).astype("float")
+
+        return cls(
+            e_true_lo=e_true[:-1],
+            e_true_hi=e_true[1:],
+            e_reco_lo=e_reco[:-1],
+            e_reco_hi=e_reco[1:],
+            data=data,
+        )
+
+    @classmethod
+    def from_hdulist(cls, hdulist, hdu1="MATRIX", hdu2="EBOUNDS"):
         """Create `EnergyDispersion` object from `~astropy.io.fits.HDUList`.
 
         Parameters
@@ -184,26 +246,32 @@ class EnergyDispersion(object):
         data = matrix_hdu.data
         header = matrix_hdu.header
 
-        pdf_matrix = np.zeros([len(data), header['DETCHANS']], dtype=np.float64)
+        pdf_matrix = np.zeros([len(data), header["DETCHANS"]], dtype=np.float64)
 
         for i, l in enumerate(data):
-            if l.field('N_GRP'):
+            if l.field("N_GRP"):
                 m_start = 0
-                for k in range(l.field('N_GRP')):
-                    pdf_matrix[i, l.field('F_CHAN')[k]: l.field(
-                        'F_CHAN')[k] + l.field('N_CHAN')[k]] = l.field(
-                        'MATRIX')[m_start:m_start + l.field('N_CHAN')[k]]
-                    m_start += l.field('N_CHAN')[k]
+                for k in range(l.field("N_GRP")):
+                    pdf_matrix[
+                        i,
+                        l.field("F_CHAN")[k] : l.field("F_CHAN")[k]
+                        + l.field("N_CHAN")[k],
+                    ] = l.field("MATRIX")[m_start : m_start + l.field("N_CHAN")[k]]
+                    m_start += l.field("N_CHAN")[k]
 
         e_reco = EnergyBounds.from_ebounds(ebounds_hdu)
         e_true = EnergyBounds.from_rmf_matrix(matrix_hdu)
 
-        return cls(e_true_lo=e_true.lower_bounds, e_true_hi=e_true.upper_bounds,
-                   e_reco_lo=e_reco.lower_bounds, e_reco_hi=e_reco.upper_bounds,
-                   data=pdf_matrix)
+        return cls(
+            e_true_lo=e_true.lower_bounds,
+            e_true_hi=e_true.upper_bounds,
+            e_reco_lo=e_reco.lower_bounds,
+            e_reco_hi=e_reco.upper_bounds,
+            data=pdf_matrix,
+        )
 
     @classmethod
-    def read(cls, filename, hdu1='MATRIX', hdu2='EBOUNDS', **kwargs):
+    def read(cls, filename, hdu1="MATRIX", hdu2="EBOUNDS"):
         """Read from file.
 
         Parameters
@@ -216,13 +284,10 @@ class EnergyDispersion(object):
             HDU containing the energy axis information, default, EBOUNDS
         """
         filename = make_path(filename)
-        hdulist = fits.open(str(filename), **kwargs)
-        try:
-            return cls.from_hdulist(hdulist, hdu1=hdu1, hdu2=hdu2)
-        except KeyError:
-            msg = 'File {} contains no HDU "{}"'.format(filename, hdu)
-            msg += '\n Available {}'.format([_.name for _ in hdulist])
-            raise ValueError(msg)
+        with fits.open(str(filename), memmap=False) as hdulist:
+            edisp = cls.from_hdulist(hdulist, hdu1=hdu1, hdu2=hdu2)
+
+        return edisp
 
     def to_hdulist(self, **kwargs):
         """Convert RMF to FITS HDU list format.
@@ -248,23 +313,26 @@ class EnergyDispersion(object):
         # http://docs.astropy.org/en/v1.0.4/io/fits/usage/unfamiliar.html
 
         table = self.to_table()
-        name = table.meta.pop('name')
+        name = table.meta.pop("name")
 
         header = fits.Header()
         header.update(table.meta)
 
         cols = table.columns
-        c0 = fits.Column(name=cols[0].name, format='E', array=cols[0],
-                         unit='{}'.format(cols[0].unit))
-        c1 = fits.Column(name=cols[1].name, format='E', array=cols[1],
-                         unit='{}'.format(cols[1].unit))
-        c2 = fits.Column(name=cols[2].name, format='I', array=cols[2])
-        c3 = fits.Column(name=cols[3].name, format='PI()', array=cols[3])
-        c4 = fits.Column(name=cols[4].name, format='PI()', array=cols[4])
-        c5 = fits.Column(name=cols[5].name, format='PE()', array=cols[5])
+        c0 = fits.Column(
+            name=cols[0].name, format="E", array=cols[0], unit="{}".format(cols[0].unit)
+        )
+        c1 = fits.Column(
+            name=cols[1].name, format="E", array=cols[1], unit="{}".format(cols[1].unit)
+        )
+        c2 = fits.Column(name=cols[2].name, format="I", array=cols[2])
+        c3 = fits.Column(name=cols[3].name, format="PI()", array=cols[3])
+        c4 = fits.Column(name=cols[4].name, format="PI()", array=cols[4])
+        c5 = fits.Column(name=cols[5].name, format="PE()", array=cols[5])
 
-        hdu = fits.BinTableHDU.from_columns([c0, c1, c2, c3, c4, c5],
-                                            header=header, name=name)
+        hdu = fits.BinTableHDU.from_columns(
+            [c0, c1, c2, c3, c4, c5], header=header, name=name
+        )
 
         ebounds = energy_axis_to_ebounds(self.e_reco.bins)
         prim_hdu = fits.PrimaryHDU()
@@ -285,7 +353,6 @@ class EnergyDispersion(object):
 
         # Make RMF type matrix
         for i, row in enumerate(self.data.data.value):
-            subsets = 1
             pos = np.nonzero(row)[0]
             borders = np.where(np.diff(pos) != 1)[0]
             # add 1 to borders for correct behaviour of np.split
@@ -312,24 +379,26 @@ class EnergyDispersion(object):
 
         table = Table()
 
-        table['ENERG_LO'] = self.e_true.lo
-        table['ENERG_HI'] = self.e_true.hi
-        table['N_GRP'] = n_grp
-        table['F_CHAN'] = f_chan
-        table['N_CHAN'] = n_chan
-        table['MATRIX'] = matrix
+        table["ENERG_LO"] = self.e_true.lo
+        table["ENERG_HI"] = self.e_true.hi
+        table["N_GRP"] = n_grp
+        table["F_CHAN"] = f_chan
+        table["N_CHAN"] = n_chan
+        table["MATRIX"] = matrix
 
-        table.meta = OrderedDict([
-            ('name', 'MATRIX'),
-            ('chantype', 'PHA'),
-            ('hduclass', 'OGIP'),
-            ('hduclas1', 'RESPONSE'),
-            ('hduclas2', 'RSP_MATRIX'),
-            ('detchans', self.e_reco.nbins),
-            ('numgrp', numgrp),
-            ('numelt', numelt),
-            ('tlmin4', 0),
-        ])
+        table.meta = OrderedDict(
+            [
+                ("name", "MATRIX"),
+                ("chantype", "PHA"),
+                ("hduclass", "OGIP"),
+                ("hduclas1", "RESPONSE"),
+                ("hduclas2", "RSP_MATRIX"),
+                ("detchans", self.e_reco.nbins),
+                ("numgrp", numgrp),
+                ("numelt", numelt),
+                ("tlmin4", 0),
+            ]
+        )
 
         return table
 
@@ -379,7 +448,7 @@ class EnergyDispersion(object):
         idx = self.e_true.find_node(e_true)
         pdf = self.data.data[idx]
 
-        # compute sum along reconstructed energy 
+        # compute sum along reconstructed energy
         # axis to determine the mean
         norm = np.sum(pdf, axis=-1)
         temp = np.sum(pdf * self.e_reco.nodes, axis=-1)
@@ -399,7 +468,9 @@ class EnergyDispersion(object):
         # for each given true energy value
         # (first axis is reconstructed energy)
         erec = self.e_reco.nodes
-        erec = np.repeat(erec, max(np.sum(mean.shape), 1)).reshape(erec.shape + mean.shape)
+        erec = np.repeat(erec, max(np.sum(mean.shape), 1)).reshape(
+            erec.shape + mean.shape
+        )
 
         # compute deviation from mean
         # (and move reconstructed energy axis to last axis)
@@ -414,7 +485,7 @@ class EnergyDispersion(object):
         return var / norm
 
     def to_sherpa(self, name):
-        """Convert to `sherpa.astro.data.DataARF`.
+        """Convert to `sherpa.astro.data.DataRMF`.
 
         Parameters
         ----------
@@ -422,18 +493,16 @@ class EnergyDispersion(object):
             Instance name
         """
         from sherpa.astro.data import DataRMF
-        from sherpa.utils import SherpaInt, SherpaUInt, SherpaFloat
+        from sherpa.utils import SherpaUInt, SherpaFloat
 
         # Need to modify RMF data
         # see https://github.com/sherpa/sherpa/blob/master/sherpa/astro/io/pyfits_backend.py#L727
 
         table = self.to_table()
-        n_grp = table['N_GRP'].data.astype(SherpaUInt)
-        f_chan = table['F_CHAN'].data
-        f_chan = np.concatenate([row for row in f_chan]).astype(SherpaUInt)
-        n_chan = table['N_CHAN'].data
-        n_chan = np.concatenate([row for row in n_chan]).astype(SherpaUInt)
-        matrix = table['MATRIX'].data
+        n_grp = table["N_GRP"].data.astype(SherpaUInt)
+        f_chan = table["F_CHAN"].data
+        n_chan = table["N_CHAN"].data
+        matrix = table["MATRIX"].data
 
         good = n_grp > 0
         matrix = matrix[good]
@@ -442,19 +511,21 @@ class EnergyDispersion(object):
 
         good = n_grp > 0
         f_chan = f_chan[good]
+        f_chan = np.concatenate([row for row in f_chan]).astype(SherpaUInt)
         n_chan = n_chan[good]
+        n_chan = np.concatenate([row for row in n_chan]).astype(SherpaUInt)
 
         return DataRMF(
             name=name,
-            energ_lo=table['ENERG_LO'].quantity.to('keV').value.astype(SherpaFloat),
-            energ_hi=table['ENERG_HI'].quantity.to('keV').value.astype(SherpaFloat),
+            energ_lo=table["ENERG_LO"].quantity.to_value("keV").astype(SherpaFloat),
+            energ_hi=table["ENERG_HI"].quantity.to_value("keV").astype(SherpaFloat),
             matrix=matrix,
             n_grp=n_grp,
             n_chan=n_chan,
             f_chan=f_chan,
             detchans=self.e_reco.nbins,
-            e_min=self.e_reco.lo.to('keV').value,
-            e_max=self.e_reco.hi.to('keV').value,
+            e_min=self.e_reco.lo.to_value("keV"),
+            e_max=self.e_reco.hi.to_value("keV"),
             offset=0,
         )
 
@@ -478,9 +549,9 @@ class EnergyDispersion(object):
         import matplotlib.pyplot as plt
         from matplotlib.colors import PowerNorm
 
-        kwargs.setdefault('cmap', 'GnBu')
+        kwargs.setdefault("cmap", "GnBu")
         norm = PowerNorm(gamma=0.5)
-        kwargs.setdefault('norm', norm)
+        kwargs.setdefault("norm", norm)
 
         ax = plt.gca() if ax is None else ax
 
@@ -492,17 +563,17 @@ class EnergyDispersion(object):
         caxes = ax.pcolormesh(x, y, z.T, **kwargs)
 
         if show_energy is not None:
-            ener_val = Quantity(show_energy).to(self.reco_energy.unit).value
-            ax.hlines(ener_val, 0, 200200, linestyles='dashed')
+            ener_val = show_energy.to_value(self.reco_energy.unit)
+            ax.hlines(ener_val, 0, 200200, linestyles="dashed")
 
         if add_cbar:
-            label = 'Probability density (A.U.)'
+            label = "Probability density (A.U.)"
             cbar = ax.figure.colorbar(caxes, ax=ax, label=label)
 
-        ax.set_xlabel('$E_\mathrm{{True}}$ [{unit}]'.format(unit=e_true.unit))
-        ax.set_ylabel('$E_\mathrm{{Reco}}$ [{unit}]'.format(unit=e_reco.unit))
-        ax.set_xscale('log')
-        ax.set_yscale('log')
+        ax.set_xlabel(r"$E_\mathrm{{True}}$ [{unit}]".format(unit=e_true.unit))
+        ax.set_ylabel(r"$E_\mathrm{{Reco}}$ [{unit}]".format(unit=e_reco.unit))
+        ax.set_xscale("log")
+        ax.set_yscale("log")
         ax.set_xlim(x.min(), x.max())
         ax.set_ylim(y.min(), y.max())
         return ax
@@ -521,18 +592,19 @@ class EnergyDispersion(object):
 
         ax = plt.gca() if ax is None else ax
 
-        x = self.e_true.nodes.to('TeV').value
+        x = self.e_true.nodes.to_value("TeV")
         y = self.get_bias(self.e_true.nodes)
 
         ax.plot(x, y, **kwargs)
-        ax.set_xlabel('$E_\mathrm{{True}}$ [TeV]')
-        ax.set_ylabel(r'($E_\mathrm{{True}} - E_\mathrm{{Reco}} / E_\mathrm{{True}}$)')
-        ax.set_xscale('log')
+        ax.set_xlabel(r"$E_\mathrm{{True}}$ [TeV]")
+        ax.set_ylabel(r"($E_\mathrm{{True}} - E_\mathrm{{Reco}} / E_\mathrm{{True}}$)")
+        ax.set_xscale("log")
         return ax
 
     def peek(self, figsize=(15, 5)):
         """Quick-look summary plot."""
         import matplotlib.pyplot as plt
+
         fig, axes = plt.subplots(nrows=1, ncols=2, figsize=figsize)
         self.plot_bias(ax=axes[0])
         self.plot_matrix(ax=axes[1])
@@ -575,8 +647,7 @@ class EnergyDispersion2D(object):
     for a given field of view offset and energy binning:
 
     >>> energy = EnergyBounds.equal_log_spacing(0.1,20,60, 'TeV')
-    >>> offset = '1.2 deg'
-    >>> edisp = edisp2d.to_energy_dispersion(offset=offset, e_reco=energy, e_true=energy)
+    >>> edisp = edisp2d.to_energy_dispersion(offset='1.2 deg', e_reco=energy, e_true=energy)
     >>> print(edisp)
     EnergyDispersion
     NDDataArray summary info
@@ -588,28 +659,41 @@ class EnergyDispersion2D(object):
     --------
     EnergyDispersion
     """
+
     default_interp_kwargs = dict(bounds_error=False, fill_value=None)
     """Default Interpolation kwargs for `~gammapy.utils.nddata.NDDataArray`. Extrapolate."""
 
-    def __init__(self, e_true_lo, e_true_hi, migra_lo, migra_hi, offset_lo,
-                 offset_hi, data, interp_kwargs=None, meta=None,):
+    def __init__(
+        self,
+        e_true_lo,
+        e_true_hi,
+        migra_lo,
+        migra_hi,
+        offset_lo,
+        offset_hi,
+        data,
+        interp_kwargs=None,
+        meta=None,
+    ):
         if interp_kwargs is None:
             interp_kwargs = self.default_interp_kwargs
         axes = [
-            BinnedDataAxis(e_true_lo, e_true_hi,
-                           interpolation_mode='log', name='e_true'),
-            BinnedDataAxis(migra_lo, migra_hi,
-                           interpolation_mode='linear', name='migra'),
-            BinnedDataAxis(offset_lo, offset_hi,
-                           interpolation_mode='linear', name='offset')
+            BinnedDataAxis(
+                e_true_lo, e_true_hi, interpolation_mode="log", name="e_true"
+            ),
+            BinnedDataAxis(
+                migra_lo, migra_hi, interpolation_mode="linear", name="migra"
+            ),
+            BinnedDataAxis(
+                offset_lo, offset_hi, interpolation_mode="linear", name="offset"
+            ),
         ]
-        self.data = NDDataArray(axes=axes, data=data,
-                                interp_kwargs=interp_kwargs)
+        self.data = NDDataArray(axes=axes, data=data, interp_kwargs=interp_kwargs)
         self.meta = OrderedDict(meta) if meta else OrderedDict()
 
     def __str__(self):
         ss = self.__class__.__name__
-        ss += '\n{}'.format(self.data)
+        ss += "\n{}".format(self.data)
         return ss
 
     @classmethod
@@ -639,11 +723,9 @@ class EnergyDispersion2D(object):
         pdf_threshold : float, optional
             Zero suppression threshold
         """
-        from scipy.special import erf
-
         e_true = EnergyBounds(e_true)
         # erf does not work with Quantities
-        true = e_true.log_centers.to('TeV').value
+        true = e_true.log_centers.to_value("TeV")
 
         true2d, migra2d = np.meshgrid(true, migra)
 
@@ -660,39 +742,54 @@ class EnergyDispersion2D(object):
 
         pdf_array = np.where(pdf_array > pdf_threshold, pdf_array, 0)
 
-        return cls(e_true[:-1], e_true[1:], migra[:-1], migra[1:],
-                   offset[:-1], offset[1:], pdf_array)
+        return cls(
+            e_true[:-1],
+            e_true[1:],
+            migra[:-1],
+            migra[1:],
+            offset[:-1],
+            offset[1:],
+            pdf_array,
+        )
 
     @classmethod
     def from_table(cls, table):
         """Create from `~astropy.table.Table`."""
-        if 'ENERG_LO' in table.colnames:
-            e_lo = table['ENERG_LO'].quantity[0]
-            e_hi = table['ENERG_HI'].quantity[0]
-        elif 'ETRUE_LO' in table.colnames:
-            e_lo = table['ETRUE_LO'].quantity[0]
-            e_hi = table['ETRUE_HI'].quantity[0]
+        if "ENERG_LO" in table.colnames:
+            e_lo = table["ENERG_LO"].quantity[0]
+            e_hi = table["ENERG_HI"].quantity[0]
+        elif "ETRUE_LO" in table.colnames:
+            e_lo = table["ETRUE_LO"].quantity[0]
+            e_hi = table["ETRUE_HI"].quantity[0]
         else:
-            raise ValueError('Invalid column names. Need "ENERG_LO/ENERG_HI" or "ETRUE_LO/ETRUE_HI"')
-        o_lo = table['THETA_LO'].quantity[0]
-        o_hi = table['THETA_HI'].quantity[0]
-        m_lo = table['MIGRA_LO'].quantity[0]
-        m_hi = table['MIGRA_HI'].quantity[0]
+            raise ValueError(
+                'Invalid column names. Need "ENERG_LO/ENERG_HI" or "ETRUE_LO/ETRUE_HI"'
+            )
+        o_lo = table["THETA_LO"].quantity[0]
+        o_hi = table["THETA_HI"].quantity[0]
+        m_lo = table["MIGRA_LO"].quantity[0]
+        m_hi = table["MIGRA_HI"].quantity[0]
 
-        matrix = table['MATRIX'].quantity[0].transpose() ## TODO Why does this need to be transposed?
-        return cls(e_true_lo=e_lo, e_true_hi=e_hi,
-                   offset_lo=o_lo, offset_hi=o_hi,
-                   migra_lo=m_lo, migra_hi=m_hi, data=matrix)
+        # TODO Why does this need to be transposed?
+        matrix = table["MATRIX"].quantity[0].transpose()
+
+        return cls(
+            e_true_lo=e_lo,
+            e_true_hi=e_hi,
+            offset_lo=o_lo,
+            offset_hi=o_hi,
+            migra_lo=m_lo,
+            migra_hi=m_hi,
+            data=matrix,
+        )
 
     @classmethod
-    def from_hdulist(cls, hdulist, hdu='edisp_2d'):
+    def from_hdulist(cls, hdulist, hdu="edisp_2d"):
         """Create from `~astropy.io.fits.HDUList`."""
-        hdu = hdulist[hdu]
-        table = fits_table_to_table(hdu)
-        return cls.from_table(table)
+        return cls.from_table(Table.read(hdulist[hdu]))
 
     @classmethod
-    def read(cls, filename, hdu='edisp_2d'):
+    def read(cls, filename, hdu="edisp_2d"):
         """Read from FITS file.
 
         Parameters
@@ -701,8 +798,10 @@ class EnergyDispersion2D(object):
             File name
         """
         filename = make_path(filename)
-        hdulist = fits.open(str(filename))
-        return cls.from_hdulist(hdulist, hdu)
+        with fits.open(str(filename), memmap=False) as hdulist:
+            edisp = cls.from_hdulist(hdulist, hdu)
+
+        return edisp
 
     def to_energy_dispersion(self, offset, e_true=None, e_reco=None):
         """Detector response R(Delta E_reco, Delta E_true)
@@ -725,8 +824,8 @@ class EnergyDispersion2D(object):
             Energy dispersion matrix
         """
         offset = Angle(offset)
-        e_true = self.data.axis('e_true').bins if e_true is None else e_true
-        e_reco = self.data.axis('e_true').bins if e_reco is None else e_reco
+        e_true = self.data.axis("e_true").bins if e_true is None else e_true
+        e_reco = self.data.axis("e_true").bins if e_reco is None else e_reco
         e_true = EnergyBounds(e_true)
         e_reco = EnergyBounds(e_reco)
 
@@ -739,9 +838,13 @@ class EnergyDispersion2D(object):
         e_lo, e_hi = e_true[:-1], e_true[1:]
         ereco_lo, ereco_hi = (e_reco[:-1], e_reco[1:])
 
-        return EnergyDispersion(e_true_lo=e_lo, e_true_hi=e_hi,
-                                e_reco_lo=ereco_lo, e_reco_hi=ereco_hi,
-                                data=data)
+        return EnergyDispersion(
+            e_true_lo=e_lo,
+            e_true_hi=e_hi,
+            e_reco_lo=ereco_lo,
+            e_reco_hi=ereco_hi,
+            data=data,
+        )
 
     def get_response(self, offset, e_true, e_reco=None, migra_step=5e-3):
         """Detector response R(Delta E_reco, E_true)
@@ -768,30 +871,30 @@ class EnergyDispersion2D(object):
         """
         e_true = Energy(e_true)
 
-        # Default: e_reco nodes = migra nodes * e_true nodes
         if e_reco is None:
+            # Default: e_reco nodes = migra nodes * e_true nodes
+            migra_axis = self.data.axis("migra")
             e_reco = EnergyBounds.from_lower_and_upper_bounds(
-                self.data.axis('migra').lo * e_true, self.data.axis('migra').hi * e_true)
-            migra = self.data.axis('migra').nodes
-        # Translate given e_reco binning to migra at bin center
+                migra_axis.lo * e_true, migra_axis.hi * e_true
+            )
         else:
+            # Translate given e_reco binning to migra at bin center
             e_reco = EnergyBounds(e_reco)
-            center = e_reco.log_centers
-            migra = center / e_true
 
         # migration value of e_reco bounds
         migra_e_reco = e_reco / e_true
 
         # Define a vector of migration with mig_step step
-        mrec_min = self.data.axis('migra').lo[0]
-        mrec_max = self.data.axis('migra').hi[-1]
+        mrec_min = self.data.axis("migra").lo[0]
+        mrec_max = self.data.axis("migra").hi[-1]
         mig_array = np.arange(mrec_min, mrec_max, migra_step)
 
         # Compute energy dispersion probability dP/dm for each element of migration array
         vals = self.data.evaluate(offset=offset, e_true=e_true, migra=mig_array)
 
         # Compute normalized cumulative sum to prepare integration
-        tmp = np.nan_to_num(np.cumsum(vals) / np.sum(vals))
+        with np.errstate(invalid="ignore"):
+            tmp = np.nan_to_num(np.cumsum(vals) / np.sum(vals))
 
         # Determine positions (bin indices) of e_reco bounds in migration array
         pos_mig = np.digitize(migra_e_reco, mig_array) - 1
@@ -804,8 +907,7 @@ class EnergyDispersion2D(object):
 
         return integral
 
-    def plot_migration(self, ax=None, offset=None, e_true=None,
-                       migra=None, **kwargs):
+    def plot_migration(self, ax=None, offset=None, e_true=None, migra=None, **kwargs):
         """Plot energy dispersion for given offset and true energy.
 
         Parameters
@@ -829,24 +931,24 @@ class EnergyDispersion2D(object):
         ax = plt.gca() if ax is None else ax
 
         if offset is None:
-            offset = Angle([1], 'deg')
+            offset = Angle([1], "deg")
         else:
             offset = np.atleast_1d(Angle(offset))
         if e_true is None:
-            e_true = Energy([0.1, 1, 10], 'TeV')
+            e_true = Energy([0.1, 1, 10], "TeV")
         else:
             e_true = np.atleast_1d(Energy(e_true))
-        migra = self.data.axis('migra').nodes if migra is None else migra
+        migra = self.data.axis("migra").nodes if migra is None else migra
 
         for ener in e_true:
             for off in offset:
                 disp = self.data.evaluate(offset=off, e_true=ener, migra=migra)
-                label = 'offset = {0:.1f}\nenergy = {1:.1f}'.format(off, ener)
+                label = "offset = {0:.1f}\nenergy = {1:.1f}".format(off, ener)
                 ax.plot(migra, disp, label=label, **kwargs)
 
-        ax.set_xlabel('$E_\mathrm{{Reco}} / E_\mathrm{{True}}$')
-        ax.set_ylabel('Probability density')
-        ax.legend(loc='upper left')
+        ax.set_xlabel(r"$E_\mathrm{{Reco}} / E_\mathrm{{True}}$")
+        ax.set_ylabel("Probability density")
+        ax.legend(loc="upper left")
 
         return ax
 
@@ -872,32 +974,36 @@ class EnergyDispersion2D(object):
         from matplotlib.colors import PowerNorm
         import matplotlib.pyplot as plt
 
-        kwargs.setdefault('cmap', 'GnBu')
-        kwargs.setdefault('norm', PowerNorm(gamma=0.5))
+        kwargs.setdefault("cmap", "GnBu")
+        kwargs.setdefault("norm", PowerNorm(gamma=0.5))
 
         ax = plt.gca() if ax is None else ax
 
         if offset is None:
-            offset = Angle([1], 'deg')
+            offset = Angle(1, "deg")
 
-        e_true = self.data.axis('e_true').bins
-        migra = self.data.axis('migra').bins
+        e_true = self.data.axis("e_true").bins
+        migra = self.data.axis("migra").bins
 
         x = e_true.value
         y = migra.value
-        z = self.data.evaluate(offset=offset, e_true=e_true, migra=migra).value
+        z = self.data.evaluate(
+            offset=offset,
+            e_true=e_true.reshape(1, -1, 1),
+            migra=migra.reshape(1, 1, -1),
+        ).value[0]
 
         caxes = ax.pcolormesh(x, y, z.T, **kwargs)
 
         if add_cbar:
-            label = 'Probability density (A.U.)'
-            cbar = ax.figure.colorbar(caxes, ax=ax, label=label)
+            label = "Probability density (A.U.)"
+            ax.figure.colorbar(caxes, ax=ax, label=label)
 
-        ax.semilogx()
-        ax.set_xlabel('$E_\mathrm{{True}}$ [{unit}]'.format(unit=e_true.unit))
-        ax.set_ylabel('$E_\mathrm{{Reco}} / E_\mathrm{{True}}$')
+        ax.set_xlabel(r"$E_\mathrm{{True}}$ [{unit}]".format(unit=e_true.unit))
+        ax.set_ylabel(r"$E_\mathrm{{Reco}} / E_\mathrm{{True}}$")
         ax.set_xlim(x.min(), x.max())
         ax.set_ylim(y.min(), y.max())
+        ax.set_xscale("log")
         return ax
 
     def peek(self, figsize=(15, 5)):
@@ -909,10 +1015,11 @@ class EnergyDispersion2D(object):
             Size of the resulting plot
         """
         import matplotlib.pyplot as plt
+
         fig, axes = plt.subplots(nrows=1, ncols=3, figsize=figsize)
         self.plot_bias(ax=axes[0])
         self.plot_migration(ax=axes[1])
-        edisp = self.to_energy_dispersion(offset='1 deg')
+        edisp = self.to_energy_dispersion(offset="1 deg")
         edisp.plot_matrix(ax=axes[2])
 
         plt.tight_layout()
@@ -921,15 +1028,15 @@ class EnergyDispersion2D(object):
         """Convert to `~astropy.table.Table`."""
         meta = self.meta.copy()
         table = Table(meta=meta)
-        table['ENERG_LO'] = self.data.axis('e_true').lo[np.newaxis]
-        table['ENERG_HI'] = self.data.axis('e_true').hi[np.newaxis]
-        table['MIGRA_LO'] = self.data.axis('migra').hi[np.newaxis]
-        table['MIGRA_HI'] = self.data.axis('migra').hi[np.newaxis]
-        table['THETA_LO'] = self.data.axis('offset').lo[np.newaxis]
-        table['THETA_HI'] = self.data.axis('offset').hi[np.newaxis]
-        table['MATRIX'] = self.data.data.T[np.newaxis]
+        table["ENERG_LO"] = self.data.axis("e_true").lo[np.newaxis]
+        table["ENERG_HI"] = self.data.axis("e_true").hi[np.newaxis]
+        table["MIGRA_LO"] = self.data.axis("migra").hi[np.newaxis]
+        table["MIGRA_HI"] = self.data.axis("migra").hi[np.newaxis]
+        table["THETA_LO"] = self.data.axis("offset").lo[np.newaxis]
+        table["THETA_HI"] = self.data.axis("offset").hi[np.newaxis]
+        table["MATRIX"] = self.data.data.T[np.newaxis]
         return table
 
-    def to_fits(self, name='ENERGY DISPERSION'):
+    def to_fits(self, name="ENERGY DISPERSION"):
         """Convert to `~astropy.io.fits.BinTable`."""
-        return table_to_fits_table(self.to_table(), name)
+        return fits.BinTableHDU(self.to_table(), name=name)
