@@ -2,23 +2,19 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 import logging
 import numpy as np
+from scipy.interpolate import UnivariateSpline, RegularGridInterpolator
 from astropy.io import fits
 from astropy.units import Quantity
 from astropy.coordinates import Angle, SkyCoord
-from astropy.convolution.utils import discretize_oversample_2D
-from ..image.models import Gauss2DPDF
+from ..utils.gauss import Gauss2DPDF
+from ..utils.scripts import make_path
 from ..utils.array import array_stats_str
 from ..utils.energy import Energy
 
-__all__ = [
-    'TablePSF',
-    'EnergyDependentTablePSF',
-]
-
+__all__ = ["TablePSF", "EnergyDependentTablePSF"]
 
 log = logging.getLogger(__name__)
 
-
 # Default PSF spline keyword arguments
 # TODO: test and document
 DEFAULT_PSF_SPLINE_KWARGS = dict(k=1, s=0)
@@ -62,14 +58,14 @@ class TablePSF(object):
 
     def __init__(self, rad, dp_domega, spline_kwargs=DEFAULT_PSF_SPLINE_KWARGS):
 
-        self._rad = Angle(rad).to('radian')
-        self._dp_domega = Quantity(dp_domega).to('sr^-1')
+        self._rad = Angle(rad).to("radian")
+        self._dp_domega = Quantity(dp_domega).to("sr^-1")
 
         assert self._rad.ndim == self._dp_domega.ndim == 1
         assert self._rad.shape == self._dp_domega.shape
 
         # Store input arrays as quantities in default internal units
-        self._dp_dr = (2 * np.pi * self._rad * self._dp_domega).to('radian^-1')
+        self._dp_dr = (2 * np.pi * self._rad * self._dp_domega).to("radian^-1")
         self._spline_kwargs = spline_kwargs
 
         self._compute_splines(spline_kwargs)
@@ -105,28 +101,29 @@ class TablePSF(object):
         width = Angle(width)
         rad = Angle(rad)
 
-        if shape == 'disk':
+        if shape == "disk":
             amplitude = 1 / (np.pi * width.radian ** 2)
             psf_value = np.where(rad < width, amplitude, 0)
-        elif shape == 'gauss':
+        elif shape == "gauss":
             gauss2d_pdf = Gauss2DPDF(sigma=width.radian)
             psf_value = gauss2d_pdf(rad.radian)
         else:
-            raise ValueError('Invalid shape: {}'.format(shape))
+            raise ValueError("Invalid shape: {}".format(shape))
 
-        psf_value = Quantity(psf_value, 'sr^-1')
+        psf_value = Quantity(psf_value, "sr^-1")
 
         return cls(rad, psf_value)
 
     def info(self):
         """Print basic info."""
-        ss = array_stats_str(self._rad.degree, 'offset')
-        ss += 'integral = {}\n'.format(self.integral())
+        ss = array_stats_str(self._rad.degree, "offset")
+        ss += "integral = {}\n".format(self.integral())
 
         for containment in [50, 68, 80, 95]:
             radius = self.containment_radius(0.01 * containment)
-            ss += ('containment radius {} deg for {}%\n'
-                   .format(radius.degree, containment))
+            ss += "containment radius {} deg for {}%\n".format(
+                radius.degree, containment
+            )
 
         return ss
 
@@ -146,57 +143,12 @@ class TablePSF(object):
         psf_value : `~astropy.units.Quantity`
             PSF value
         """
-        center = SkyCoord(0, 0, unit='radian')
+        center = SkyCoord(0, 0, unit="radian")
         point = SkyCoord(lon, lat)
         rad = center.separation(point)
         return self.evaluate(rad)
 
-    def kernel(self, reference, rad_max, normalize=True,
-               discretize_model_kwargs=dict(factor=10)):
-        """
-        Make a 2-dimensional kernel image.
-
-        The kernel image is evaluated on a cartesian grid defined by the
-        reference sky image.
-
-        Parameters
-        ----------
-        reference : `~gammapy.image.SkyImage` or `~gammapy.cube.SkyCube`
-            Reference sky image or sky cube defining the spatial grid.
-        rad_max : `~astropy.coordinates.Angle`
-            Radial size of the kernel
-        normalize : bool
-            Whether to normalize the kernel.
-
-        Returns
-        -------
-        kernel : `~astropy.units.Quantity`
-            Kernel 2D image of Quantities
-        """
-        from ..cube import SkyCube
-        rad_max = Angle(rad_max)
-
-        if isinstance(reference, SkyCube):
-            reference = reference.sky_image_ref
-
-        pixel_size = reference.wcs_pixel_scale()[0]
-
-        def _model(x, y):
-            """Model in the appropriate format for discretize_model."""
-            rad = np.sqrt(x * x + y * y) * pixel_size
-            return self.evaluate(rad)
-
-        npix = int(rad_max.radian / pixel_size.radian)
-        pix_range = (-npix, npix + 1)
-
-        kernel = discretize_oversample_2D(_model, x_range=pix_range, y_range=pix_range,
-                                          **discretize_model_kwargs)
-        if normalize:
-            kernel = kernel / kernel.sum()
-
-        return kernel
-
-    def evaluate(self, rad, quantity='dp_domega'):
+    def evaluate(self, rad, quantity="dp_domega"):
         r"""Evaluate PSF.
 
         The following PSF quantities are available:
@@ -226,14 +178,14 @@ class TablePSF(object):
         shape = rad.shape
         x = np.array(rad.radian).flat
 
-        if quantity == 'dp_domega':
+        if quantity == "dp_domega":
             y = self._dp_domega_spline(x)
-            unit = 'sr^-1'
-        elif quantity == 'dp_dr':
+            unit = "sr^-1"
+        elif quantity == "dp_dr":
             y = self._dp_dr_spline(x)
-            unit = 'radian^-1'
+            unit = "radian^-1"
         else:
-            ss = 'Invalid quantity: {}\n'.format(quantity)
+            ss = "Invalid quantity: {}\n".format(quantity)
             ss += "Choose one of: 'dp_domega', 'dp_dr'"
             raise ValueError(ss)
 
@@ -285,7 +237,7 @@ class TablePSF(object):
             Containment radius angle
         """
         rad = self._ppf_spline(fraction)
-        return Angle(rad, 'radian').to('deg')
+        return Angle(rad, "radian").to("deg")
 
     def normalize(self):
         """Normalize PSF to unit integral.
@@ -297,10 +249,10 @@ class TablePSF(object):
 
         self._dp_dr /= integral
 
-        # Don't divide by 0
-        EPS = 1e-6
-        rad = np.clip(self._rad.radian, EPS, None)
-        rad = Quantity(rad, 'radian')
+        # Clip to small positive number to avoid divide by 0
+        rad = np.clip(self._rad.radian, 1e-6, None)
+
+        rad = Quantity(rad, "radian")
         self._dp_domega = self._dp_dr / (2 * np.pi * rad)
         self._compute_splines(self._spline_kwargs)
 
@@ -325,27 +277,28 @@ class TablePSF(object):
         self._rad *= factor
         # We define broadening such that self._dp_domega remains the same
         # so we only have to re-compute self._dp_dr and the slines here.
-        self._dp_dr = (2 * np.pi * self._rad * self._dp_domega).to('radian^-1')
+        self._dp_dr = (2 * np.pi * self._rad * self._dp_domega).to("radian^-1")
         self._compute_splines(self._spline_kwargs)
 
         if normalize:
             self.normalize()
 
-    def plot_psf_vs_rad(self, ax=None, quantity='dp_domega', **kwargs):
+    def plot_psf_vs_rad(self, ax=None, quantity="dp_domega", **kwargs):
         """Plot PSF vs radius.
 
         TODO: describe PSF ``quantity`` argument in a central place and link to it from here.
         """
         import matplotlib.pyplot as plt
+
         ax = plt.gca() if ax is None else ax
 
-        x = self._rad.to('deg')
+        x = self._rad.to("deg")
         y = self.evaluate(self._rad, quantity)
 
         ax.plot(x.value, y.value, **kwargs)
         ax.loglog()
-        ax.set_xlabel('Radius ({})'.format(x.unit))
-        ax.set_ylabel('PSF ({})'.format(y.unit))
+        ax.set_xlabel("Radius ({})".format(x.unit))
+        ax.set_ylabel("PSF ({})".format(y.unit))
 
     def _compute_splines(self, spline_kwargs=DEFAULT_PSF_SPLINE_KWARGS):
         """Compute two splines representing the PSF.
@@ -356,8 +309,6 @@ class TablePSF(object):
         * `_cdf_spline` is used to compute integral and for normalisation.
         * `_ppf_spline` is used to compute containment radii.
         """
-        from scipy.interpolate import UnivariateSpline
-
         # Compute spline and normalize.
         x, y = self._rad.value, self._dp_domega.value
         self._dp_domega_spline = UnivariateSpline(x, y, **spline_kwargs)
@@ -375,7 +326,7 @@ class TablePSF(object):
         # Here's a discussion on methods to compute the ppf
         # http://mail.scipy.org/pipermail/scipy-user/2010-May/025237.html
         y = self._rad.value
-        x = self.integral(Angle(0, 'rad'), self._rad)
+        x = self.integral(Angle(0, "rad"), self._rad)
 
         # Since scipy 1.0 the UnivariateSpline requires that x is strictly increasing
         # So only keep nodes where this is the case (and always keep the first one):
@@ -391,7 +342,7 @@ class TablePSF(object):
 
     def _rad_clip(self, rad):
         """Clip to radius support range, because spline extrapolation is unstable."""
-        rad = Angle(rad, 'radian').radian
+        rad = Angle(rad, "radian").radian
         rad = np.clip(rad, 0, self._rad[-1].radian)
         return rad
 
@@ -415,39 +366,41 @@ class EnergyDependentTablePSF(object):
 
     def __init__(self, energy, rad, exposure=None, psf_value=None):
 
-        self.energy = Quantity(energy).to('GeV')
-        self.rad = Quantity(rad).to('radian')
+        self.energy = Quantity(energy).to("GeV")
+        self.rad = Quantity(rad).to("radian")
         if exposure is None:
-            self.exposure = Quantity(np.ones(len(energy)), 'cm^2 s')
+            self.exposure = Quantity(np.ones(len(energy)), "cm^2 s")
         else:
-            self.exposure = Quantity(exposure).to('cm^2 s')
+            self.exposure = Quantity(exposure).to("cm^2 s")
 
         if psf_value is None:
-            self.psf_value = Quantity(np.zeros(len(energy), len(rad)), 'sr^-1')
+            self.psf_value = Quantity(np.zeros(len(energy), len(rad)), "sr^-1")
         else:
-            self.psf_value = Quantity(psf_value).to('sr^-1')
+            self.psf_value = Quantity(psf_value).to("sr^-1")
 
         # Cache for TablePSF at each energy ... only computed when needed
         self._table_psf_cache = [None] * len(self.energy)
 
     def __str__(self):
-        ss = 'EnergyDependentTablePSF\n'
-        ss += '-----------------------\n'
-        ss += '\nAxis info:\n'
-        ss += '  ' + array_stats_str(self.rad.to('deg'), 'rad')
-        ss += '  ' + array_stats_str(self.energy, 'energy')
+        ss = "EnergyDependentTablePSF\n"
+        ss += "-----------------------\n"
+        ss += "\nAxis info:\n"
+        ss += "  " + array_stats_str(self.rad.to("deg"), "rad")
+        ss += "  " + array_stats_str(self.energy, "energy")
         # ss += '  ' + array_stats_str(self.exposure, 'exposure')
 
         # ss += 'integral = {}\n'.format(self.integral())
 
-        ss += '\nContainment info:\n'
+        ss += "\nContainment info:\n"
         # Print some example containment radii
         fractions = [0.68, 0.95]
-        energies = Quantity([10, 100], 'GeV')
+        energies = Quantity([10, 100], "GeV")
         for fraction in fractions:
             rads = self.containment_radius(energies=energies, fraction=fraction)
             for energy, rad in zip(energies, rads):
-                ss += '  ' + '{}% containment radius at {:3.0f}: {:.2f}\n'.format(100 * fraction, energy, rad)
+                ss += "  " + "{}% containment radius at {:3.0f}: {:.2f}\n".format(
+                    100 * fraction, energy, rad
+                )
         return ss
 
     @classmethod
@@ -459,10 +412,10 @@ class EnergyDependentTablePSF(object):
         hdu_list : `~astropy.io.fits.HDUList`
             HDU list with ``THETA`` and ``PSF`` extensions.
         """
-        rad = Angle(hdu_list['THETA'].data['Theta'], 'deg')
-        energy = Quantity(hdu_list['PSF'].data['Energy'], 'MeV')
-        exposure = Quantity(hdu_list['PSF'].data['Exposure'], 'cm^2 s')
-        psf_value = Quantity(hdu_list['PSF'].data['PSF'], 'sr^-1')
+        rad = Angle(hdu_list["THETA"].data["Theta"], "deg")
+        energy = Quantity(hdu_list["PSF"].data["Energy"], "MeV")
+        exposure = Quantity(hdu_list["PSF"].data["Exposure"], "cm^2 s")
+        psf_value = Quantity(hdu_list["PSF"].data["PSF"], "sr^-1")
 
         return cls(energy, rad, exposure, psf_value)
 
@@ -477,10 +430,10 @@ class EnergyDependentTablePSF(object):
         # TODO: write HEADER keywords as gtpsf
 
         data = self.rad
-        theta_hdu = fits.BinTableHDU(data=data, name='Theta')
+        theta_hdu = fits.BinTableHDU(data=data, name="Theta")
 
         data = [self.energy, self.exposure, self.psf_value]
-        psf_hdu = fits.BinTableHDU(data=data, name='PSF')
+        psf_hdu = fits.BinTableHDU(data=data, name="PSF")
 
         hdu_list = fits.HDUList([theta_hdu, psf_hdu])
         return hdu_list
@@ -494,8 +447,11 @@ class EnergyDependentTablePSF(object):
         filename : str
             File name
         """
-        hdu_list = fits.open(filename)
-        return cls.from_fits(hdu_list)
+        filename = str(make_path(filename))
+        with fits.open(filename, memmap=False) as hdulist:
+            psf = cls.from_fits(hdulist)
+
+        return psf
 
     def write(self, *args, **kwargs):
         """Write to FITS file.
@@ -524,25 +480,26 @@ class EnergyDependentTablePSF(object):
         if interp_kwargs is None:
             interp_kwargs = dict(bounds_error=False, fill_value=None)
 
-        from scipy.interpolate import RegularGridInterpolator
         if energy is None:
             energy = self.energy
         if rad is None:
             rad = self.rad
-        energy = Energy(energy).to('TeV')
-        rad = Angle(rad).to('deg')
-        energy_bin = self.energy.to('TeV')
-        rad_bin = self.rad.to('deg')
+        energy = Energy(energy).to("TeV")
+        rad = Angle(rad).to("deg")
+        energy_bin = self.energy.to("TeV")
+        rad_bin = self.rad.to("deg")
         points = (energy_bin, rad_bin)
-        interpolator = RegularGridInterpolator(points, self.psf_value, **interp_kwargs)
-        energy_grid, rad_grid = np.meshgrid(energy.value, rad.value, indexing='ij')
+        interpolator = RegularGridInterpolator(
+            points, self.psf_value.value, **interp_kwargs
+        )
+        energy_grid, rad_grid = np.meshgrid(energy.value, rad.value, indexing="ij")
         shape = energy_grid.shape
         pix_coords = np.column_stack([energy_grid.flat, rad_grid.flat])
         data_interp = interpolator(pix_coords)
         return Quantity(data_interp.reshape(shape), self.psf_value.unit)
 
     def table_psf_at_energy(self, energy, interp_kwargs=None, **kwargs):
-        """Evaluate the `EnergyOffsetArray` at one given energy.
+        """Create `~gammapy.irf.TablePSF` at one given energy.
 
         Parameters
         ----------
@@ -553,49 +510,15 @@ class EnergyDependentTablePSF(object):
 
         Returns
         -------
-        table : `~astropy.table.Table`
-            Table with two columns: offset, value
+        psf : `~gammapy.irf.TablePSF`
+            Table PSF
         """
         psf_value = self.evaluate(energy, None, interp_kwargs)[0, :]
-        table_psf = TablePSF(self.rad, psf_value, **kwargs)
-        return table_psf
+        return TablePSF(self.rad, psf_value, **kwargs)
 
-    def kernels(self, cube, rad_max, **kwargs):
-        """
-        Make a set of 2D kernel images, representing the PSF at different energies.
-
-        The kernel image is evaluated on the spatial and energy grid defined by
-        the reference sky cube.
-
-        Parameters
-        ----------
-        cube : `~gammapy.cube.SkyCube`
-            Reference sky cube.
-        rad_max `~astropy.coordinates.Angle`
-            PSF kernel size
-        kwargs : dict
-            Keyword arguments passed to `EnergyDependentTablePSF.table_psf_in_energy_band()`.
-
-        Returns
-        -------
-        kernels : list of `~numpy.ndarray`
-            List of 2D convolution kernels.
-        """
-        energies = cube.energies(mode='edges')
-
-        kernels = []
-        for emin, emax in zip(energies[:-1], energies[1:]):
-            energy_band = Quantity([emin, emax])
-            try:
-                psf = self.table_psf_in_energy_band(energy_band, **kwargs)
-                kernel = psf.kernel(cube.sky_image_ref, rad_max=rad_max)
-            except ValueError:
-                kernel = np.nan * np.ones((1, 1))  # Dummy, means "no kernel available"
-            kernels.append(kernel)
-        return kernels
-
-    def table_psf_in_energy_band(self, energy_band, spectral_index=2,
-                                 spectrum=None, **kwargs):
+    def table_psf_in_energy_band(
+        self, energy_band, spectral_index=2, spectrum=None, **kwargs
+    ):
         """Average PSF in a given energy band.
 
         Expected counts in sub energy bands given the given exposure
@@ -616,7 +539,9 @@ class EnergyDependentTablePSF(object):
             Table PSF
         """
         if spectrum is None:
-            def spectrum(energy):
+            # This is a false positive error from pylint
+            # See https://github.com/PyCQA/pylint/issues/2410#issuecomment-415026690
+            def spectrum(energy):  # pylint:disable=function-redefined
                 return (energy / energy_band[0]) ** (-spectral_index)
 
         # TODO: warn if `energy_band` is outside available data.
@@ -700,7 +625,7 @@ class EnergyDependentTablePSF(object):
 
     def info(self):
         """Print basic info"""
-        print(self.__str__)
+        print(str(self))
 
     def plot_psf_vs_rad(self, energies=[1e4, 1e5, 1e6]):
         """Plot PSF vs radius.
@@ -710,12 +635,13 @@ class EnergyDependentTablePSF(object):
         TODO
         """
         import matplotlib.pyplot as plt
+
         plt.figure(figsize=(6, 4))
 
         for energy in energies:
             energy_index = self._energy_index(energy)
             psf = self.psf_value[energy_index, :]
-            label = '{} GeV'.format(1e-3 * energy)
+            label = "{} GeV".format(1e-3 * energy)
             x = np.hstack([-self.rad[::-1], self.rad])
             y = 1e-6 * np.hstack([psf[::-1], psf])
             plt.plot(x, y, lw=2, label=label)
@@ -723,37 +649,39 @@ class EnergyDependentTablePSF(object):
         # plt.loglog()
         plt.legend()
         plt.xlim(-0.2, 0.5)
-        plt.xlabel('Offset (deg)')
-        plt.ylabel('PSF (1e-6 sr^-1)')
+        plt.xlabel("Offset (deg)")
+        plt.ylabel("PSF (1e-6 sr^-1)")
         plt.tight_layout()
 
-    def plot_containment_vs_energy(self, ax=None, fractions=[0.63, 0.8, 0.95], **kwargs):
+    def plot_containment_vs_energy(
+        self, ax=None, fractions=[0.63, 0.8, 0.95], **kwargs
+    ):
         """Plot containment versus energy."""
         import matplotlib.pyplot as plt
 
         ax = plt.gca() if ax is None else ax
 
-        energy = Energy.equal_log_spacing(
-            self.energy.min(), self.energy.max(), 10)
+        energy = Energy.equal_log_spacing(self.energy.min(), self.energy.max(), 10)
 
         for fraction in fractions:
             rad = self.containment_radius(energy, fraction)
-            label = '{:.1f}% Containment'.format(100 * fraction)
+            label = "{:.1f}% Containment".format(100 * fraction)
             ax.plot(energy.value, rad.value, label=label, **kwargs)
 
         ax.semilogx()
-        ax.legend(loc='best')
-        ax.set_xlabel('Energy (GeV)')
-        ax.set_ylabel('Containment radius (deg)')
+        ax.legend(loc="best")
+        ax.set_xlabel("Energy (GeV)")
+        ax.set_ylabel("Containment radius (deg)")
 
     def plot_exposure_vs_energy(self):
         """Plot exposure versus energy."""
         import matplotlib.pyplot as plt
+
         plt.figure(figsize=(4, 3))
-        plt.plot(self.energy, self.exposure, color='black', lw=3)
+        plt.plot(self.energy, self.exposure, color="black", lw=3)
         plt.semilogx()
-        plt.xlabel('Energy (MeV)')
-        plt.ylabel('Exposure (cm^2 s)')
+        plt.xlabel("Energy (MeV)")
+        plt.ylabel("Exposure (cm^2 s)")
         plt.xlim(1e4 / 1.3, 1.3 * 1e6)
         plt.ylim(0, 1.5e11)
         plt.tight_layout()
@@ -778,9 +706,8 @@ class EnergyDependentTablePSF(object):
             PSF value array
         """
         psf_values = self.psf_value[energy_index, :].flatten().copy()
-        where_are_NaNs = np.isnan(psf_values)
         # When the PSF Table is not filled (with nan), the psf estimation at a given energy crashes
-        psf_values[where_are_NaNs] = 0
+        psf_values[np.isnan(psf_values)] = 0
         return psf_values
 
     def _get_1d_table_psf(self, energy_index, **kwargs):
