@@ -2,8 +2,8 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 import warnings
 import logging
-import click
 import sys
+import click
 from .. import version
 from ..extern.pathlib import Path
 
@@ -20,6 +20,9 @@ def print_version(ctx, param, value):
 # http://click.pocoo.org/5/documentation/#help-parameter-customization
 CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
 
+# https://click.palletsprojects.com/en/5.x/python3/#unicode-literals
+click.disable_unicode_literals_warning = True
+
 
 @click.group("gammapy", context_settings=CONTEXT_SETTINGS)
 @click.option(
@@ -45,7 +48,7 @@ def cli(log_level, ignore_warnings):
     Use ``--help`` to see available sub-commands, as well as the available
     arguments and options for each sub-command.
 
-    For further information, see http://gammapy.org/ and http://docs.gammapy.org/
+    For further information, see https://gammapy.org/ and https://docs.gammapy.org/
 
     \b
     Examples
@@ -102,10 +105,8 @@ def cli_jupyter(ctx, src):
     """
     Perform a series of actions on Jupyter notebooks.
 
-    The chosen action is applied for every Jupyter notebook present in the
-    current working directory. Option --file allows to chose a single file,
-    while option --fold allows to choose a different folder to scan. These
-    options are mutually exclusive, only one is allowed.
+    The chosen action is applied by default for every Jupyter notebook present
+    in the current working directory.
 
     \b
     Examples
