@@ -1,9 +1,8 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 from __future__ import absolute_import, division, print_function, unicode_literals
 from numpy.testing import assert_allclose
-from ...utils.testing import requires_data, run_cli
+from ...utils.testing import requires_data, run_cli, assert_wcs_allclose
 from ...maps import Map
-from ...utils.testing import assert_wcs_allclose
 from ..main import cli
 
 
