@@ -1,10 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-"""
-Statistics utility functions and classes.
-"""
+"""Statistics."""
 from .significance import *
 from .data import *
 from .feldman_cousins import *
 from .fit_statistics import *
 from .poisson import *
-from .utils import *
