@@ -44,7 +44,7 @@ def get_resampled_wcs(wcs, factor, downsampled):
     wcs = wcs.deepcopy()
 
     if not downsampled:
-        factor = 1. / factor
+        factor = 1.0 / factor
 
     wcs.wcs.cdelt *= factor
     wcs.wcs.crpix = (wcs.wcs.crpix - 0.5) / factor + 0.5
@@ -101,11 +101,11 @@ def linear_wcs_to_arrays(wcs, nbins_x, nbins_y):
 
     # set small values (compared to delta (i.e. step)) to 0
     for i in np.arange(len(bin_edges_x)):
-        if np.abs(bin_edges_x[i] / delta_x) < 1.e-10:
-            bin_edges_x[i] = Angle(0., unit_x)
+        if np.abs(bin_edges_x[i] / delta_x) < 1.0e-10:
+            bin_edges_x[i] = Angle(0.0, unit_x)
     for i in np.arange(len(bin_edges_y)):
-        if np.abs(bin_edges_y[i] / delta_y) < 1.e-10:
-            bin_edges_y[i] = Angle(0., unit_y)
+        if np.abs(bin_edges_y[i] / delta_y) < 1.0e-10:
+            bin_edges_y[i] = Angle(0.0, unit_y)
 
     return bin_edges_x, bin_edges_y
 
@@ -154,13 +154,13 @@ def linear_arrays_to_wcs(name_x, name_y, bin_edges_x, bin_edges_y):
     delta_y = (range_y[1] - range_y[0]) / nbins_y
     wcs.wcs.ctype = [name_x, name_y]
     wcs.wcs.cunit = [unit_x, unit_y]
-    wcs.wcs.cdelt = [delta_x.to(unit_x).value, delta_y.to(unit_y).value]
+    wcs.wcs.cdelt = [delta_x.to_value(unit_x), delta_y.to_value(unit_y)]
     # ref as lower left corner (start of (X, Y) bin coordinates)
     # coordinate start at pix = 0.5
     wcs.wcs.crpix = [0.5, 0.5]
     wcs.wcs.crval = [
-        (bin_edges_x[0] + (wcs.wcs.crpix[0] - 0.5) * delta_x).to(unit_x).value,
-        (bin_edges_y[0] + (wcs.wcs.crpix[1] - 0.5) * delta_y).to(unit_y).value,
+        (bin_edges_x[0] + (wcs.wcs.crpix[0] - 0.5) * delta_x).to_value(unit_x),
+        (bin_edges_y[0] + (wcs.wcs.crpix[1] - 0.5) * delta_y).to_value(unit_y),
     ]
 
     return wcs
